'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

function styleInject(css, ref) {
  if (ref === void 0) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') {
    return;
  }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css = ".rh-form-field {\n  margin-bottom: 20px;\n  display: flex;\n  align-items: flex-start;\n  min-height: 32px;\n}\n.rh-form-field__header {\n  margin-right: 20px;\n  display: flex;\n  justify-content: flex-end;\n  align-items: center;\n  font-size: 14px;\n  text-align: right;\n  height: 32px;\n  width: 120px;\n}\n.rh-form-field__header::before {\n  display: none;\n  content: '*';\n  color: red;\n}\n.rh-form-field__container {\n  position: relative;\n  flex: 1;\n  margin-top: auto;\n  margin-bottom: auto;\n}\n.rh-form-field__message {\n  position: absolute;\n  top: 100%;\n  left: 0;\n  font-size: 12px;\n  color: red;\n}\n.rh-form-field__footer {\n  display: flex;\n  justify-content: flex-end;\n  align-items: center;\n  height: 32px;\n  margin-left: 10px;\n  font-size: 14px;\n}\n.rh-form-field .rh-form-field {\n  margin-bottom: 0;\n}\n.rh-form-field.rh-form-field--inline {\n  display: inline-flex;\n}\n.rh-form-field.rh-form-field--compact {\n  margin-bottom: 0;\n}\n.rh-form-field.rh-form-field--compact .rh-form-field__message {\n  display: none;\n}\n.rh-form-field.rh-form-field--required .rh-form-field__header::before {\n  display: inline;\n  content: '*';\n  color: red;\n}\n";
styleInject(css);

const FormStoreContext = React__default.createContext(undefined);

function useFieldChange(store, name, onChange) {
  React.useEffect(() => {
    if (!name || !store) return;
    return store.subscribe(n => {
      if (name === '*' || n === name || n === '*') {
        onChange(name);
      }
    });
  }, [name, store]);
}

const FormOptionsContext = React__default.createContext({});

function isObject(obj) {
  return obj !== null && typeof obj === 'object';
}
function deepGet(obj, path) {
  const parts = path.split('.');
  const length = parts.length;

  for (let i = 0; i < length; i++) {
    if (!isObject(obj)) return undefined;
    obj = obj[parts[i]];
  }

  return obj;
}
function deepSet(obj, path, value) {
  if (!isObject(obj)) return obj;
  const root = obj;
  const parts = path.split('.');
  const length = parts.length;

  for (let i = 0; i < length; i++) {
    const p = parts[i];

    if (i === length - 1) {
      obj[p] = value;
    } else if (!isObject(obj[p])) {
      obj[p] = {};
    }

    obj = obj[p];
  }

  return root;
}
function deepCopy(target) {
  const type = typeof target;

  if (target === null || type === 'boolean' || type === 'number' || type === 'string') {
    return target;
  }

  if (target instanceof Date) {
    return new Date(target.getTime());
  }

  if (Array.isArray(target)) {
    return target.map(o => deepCopy(o));
  }

  if (typeof target === 'object') {
    const obj = {};

    for (let key in target) {
      obj[key] = deepCopy(target[key]);
    }

    return obj;
  }

  return undefined;
}
function getPropName(valueProp, type) {
  return typeof valueProp === 'function' ? valueProp(type) : valueProp;
}
function getValueFromEvent(...args) {
  const e = args[0];
  return e && e.target ? e.target.type === 'checkbox' ? e.target.checked : e.target.value : e;
}

function FormField(props) {
  const {
    className,
    label,
    name,
    valueProp = 'value',
    valueGetter = getValueFromEvent,
    suffix,
    children,
    ...restProps
  } = props;
  const store = React.useContext(FormStoreContext);
  const options = React.useContext(FormOptionsContext);
  const [value, setValue] = React.useState(name && store ? store.get(name) : undefined);
  const [error, setError] = React.useState(name && store ? store.error(name) : undefined);
  const onChange = React.useCallback((...args) => name && store && store.set(name, valueGetter(...args)), [name, store, valueGetter]);
  useFieldChange(store, name, () => {
    setValue(store.get(name));
    setError(store.error(name));
  });
  let child = children;

  if (name && store && React.isValidElement(child)) {
    const prop = getPropName(valueProp, child && child.type);
    const childProps = {
      [prop]: value,
      onChange
    };
    child = React.cloneElement(child, childProps);
  }

  const {
    inline,
    compact,
    required,
    labelWidth,
    gutter,
    errorClassName
  } = { ...options,
    ...restProps
  };
  const classNames = [classes.field, inline ? classes.inline : '', compact ? classes.compact : '', required ? classes.required : '', error ? classes.error : '', className ? className : '', error ? errorClassName : ''].join('');
  const headerStyle = {
    width: labelWidth,
    marginRight: gutter
  };
  return React__default.createElement("div", {
    className: classNames
  }, label !== undefined && React__default.createElement("div", {
    className: classes.header,
    style: headerStyle
  }, label), React__default.createElement("div", {
    className: classes.container
  }, React__default.createElement("div", {
    className: classes.control
  }, child), React__default.createElement("div", {
    className: classes.message
  }, error)), suffix !== undefined && React__default.createElement("div", {
    className: classes.footer
  }, suffix));
}
const classes = {
  field: 'rh-form-field ',
  inline: 'rh-form-field--inline ',
  compact: 'rh-form-field--compact ',
  required: 'rh-form-field--required ',
  error: 'rh-form-field--error ',
  header: 'rh-form-field__header',
  container: 'rh-form-field__container',
  control: 'rh-form-field__control',
  message: 'rh-form-field__message',
  footer: 'rh-form-field__footer'
};

function FormItem(props) {
  const {
    name,
    valueProp = 'value',
    valueGetter = getValueFromEvent,
    children
  } = props;
  const store = React.useContext(FormStoreContext);
  const options = React.useContext(FormOptionsContext);
  const [value, setValue] = React.useState(name && store ? store.get(name) : undefined);
  const [error, setError] = React.useState(name && store ? store.error(name) : undefined);
  const onChange = React.useCallback((...args) => name && store && store.set(name, valueGetter(...args)), [name, store, valueGetter]);
  useFieldChange(store, name, () => {
    setValue(store.get(name));
    setError(store.error(name));
  });
  let child = children;

  if (name && store && React.isValidElement(child)) {
    const {
      errorClassName
    } = options;
    const prop = getPropName(valueProp, child && child.type);
    let className = child.props && child.props.className || '';
    if (error) className += ' ' + errorClassName;
    const childProps = {
      className,
      [prop]: value,
      onChange
    };
    child = React.cloneElement(child, childProps);
  }

  return child;
}

function Form(props) {
  const {
    className = '',
    children,
    store,
    onSubmit,
    onReset,
    ...options
  } = props;
  const classNames = 'rh-form ' + className;
  return React__default.createElement(FormStoreContext.Provider, {
    value: store
  }, React__default.createElement(FormOptionsContext.Provider, {
    value: options
  }, React__default.createElement("form", {
    className: classNames,
    onSubmit: onSubmit,
    onReset: onReset
  }, children)));
}
Form.Field = FormField;
Form.Item = FormItem;

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

class FormStore {
  constructor(values = {}, rules = {}) {
    _defineProperty(this, "initialValues", void 0);

    _defineProperty(this, "listeners", []);

    _defineProperty(this, "values", void 0);

    _defineProperty(this, "rules", void 0);

    _defineProperty(this, "errors", {});

    this.initialValues = values;
    this.values = deepCopy(values);
    this.rules = rules;
    this.get = this.get.bind(this);
    this.set = this.set.bind(this);
    this.reset = this.reset.bind(this);
    this.error = this.error.bind(this);
    this.validate = this.validate.bind(this);
    this.subscribe = this.subscribe.bind(this);
  }

  notify(name) {
    this.listeners.forEach(listener => listener(name));
  }

  get(name) {
    return name === undefined ? { ...this.values
    } : deepGet(this.values, name);
  }

  async set(name, value, validate = true) {
    if (typeof name === 'string') {
      deepSet(this.values, name, value);
      this.notify(name);

      if (validate) {
        await this.validate(name);
        this.notify(name);
      }
    } else if (name) {
      await Promise.all(Object.keys(name).map(n => this.set(n, name[n])));
    }
  }

  reset() {
    this.errors = {};
    this.values = deepCopy(this.initialValues);
    this.notify('*');
  }

  error(...args) {
    let [name, value] = args;
    if (args.length === 0) return this.errors;

    if (typeof name === 'number') {
      name = Object.keys(this.errors)[name];
    }

    if (args.length === 2) {
      if (value === undefined) {
        delete this.errors[name];
      } else {
        this.errors[name] = value;
      }
    }

    return this.errors[name];
  }

  async validate(name) {
    if (name === undefined) {
      await Promise.all(Object.keys(this.rules).map(n => this.validate(n)));
      this.notify('*');
      const message = this.error(0);
      const error = message === undefined ? undefined : new Error(message);
      return [error, this.get()];
    } else {
      const validator = this.rules[name];
      const value = this.get(name);
      const error = await executeValidator(validator, value, this.values);
      this.error(name, error && error.message);
      return [error, value];
    }
  }

  subscribe(listener) {
    this.listeners.push(listener);
    return () => {
      const index = this.listeners.indexOf(listener);
      if (index > -1) this.listeners.splice(index, 1);
    };
  }

}

async function executeValidator(validator, value, values) {
  if (validator) {
    const result = await validator(value, values);

    if (result !== true && result !== undefined) {
      return new Error(typeof result === 'string' ? result : '');
    }
  }

  return undefined;
}

function useFormStore(values = {}, rules = {}) {
  return React.useMemo(() => new FormStore(values, rules), []);
}

function useFormChange(store, onChange) {
  React.useEffect(() => {
    if (!store) return;
    return store.subscribe(n => {
      onChange(n);
    });
  }, [store]);
}

exports.Form = Form;
exports.FormField = FormField;
exports.FormItem = FormItem;
exports.FormStore = FormStore;
exports.useFieldChange = useFieldChange;
exports.useFormChange = useFormChange;
exports.useFormStore = useFormStore;
//# sourceMappingURL=index.js.map
